legistar_host = 'hattiesburg.legistar.com'

DEPARTMENTS_URL = 'https://hattiesburg.legistar.com/Departments.aspx'

PEOPLE_URL = 'https://hattiesburg.legistar.com/People.aspx'


RSS_FEED_URL = 'https://hattiesburg.legistar.com/Feed.ashx?M=Calendar&ID=266278&GUID=a8ed52ea-0528-4c9b-87a7-211c56b0c6d9&Mode=All&Title=City+of+Hattiesburg%2c+MS+-+Calendar+(All)' # noqa

AMENDMENTS_TO_AGENDA_DEPT = 15836

AGENDA_ITEM_ORDER = ['presentation', 'policy', 'routine', 'unknown']

RSS_THIS_MONTH = 'https://hattiesburg.legistar.com/Feed.ashx?M=Calendar&ID=517515&GUID=f9880525-7349-4f51-b375-ac3ded1ccf13&Mode=This+Month&Title=City+of+Hattiesburg,+MS+-+Calendar+(This+Month)' # noqa

RSS_YEAR_2011 = 'https://hattiesburg.legistar.com/Feed.ashx?M=Calendar&ID=517521&GUID=519a4642-ea34-4238-8c5d-68ba5056030f&Mode=2011&Title=City+of+Hattiesburg,+MS+-+Calendar+(2011)' # noqa

RSS_YEAR_2012 = 'https://hattiesburg.legistar.com/Feed.ashx?M=Calendar&ID=517522&GUID=ba87113e-9748-46cc-bed7-09923264420b&Mode=2012&Title=City+of+Hattiesburg,+MS+-+Calendar+(2012)' # noqa

RSS_YEAR_2013 = 'https://hattiesburg.legistar.com/Feed.ashx?M=Calendar&ID=899558&GUID=c71a2185-d86d-484e-b6bc-d7edbfaf07fe&Mode=2013&Title=City+of+Hattiesburg%2c+MS+-+Calendar+%282013%29' # noqa

RSS_YEAR_2014 = 'https://hattiesburg.legistar.com/Feed.ashx?M=Calendar&ID=1304070&GUID=c7f1c7cf-0401-487e-85d8-b25ae27bb0a6&Mode=2014&Title=City+of+Hattiesburg%2c+MS+-+Calendar+%282014%29' # noqa

RSS_YEAR_2015 = 'https://hattiesburg.legistar.com/Feed.ashx?M=Calendar&ID=1800408&GUID=4a65e270-dca1-487b-a50a-775a9e595ecd&Mode=2015&Title=City+of+Hattiesburg%2c+MS+-+Calendar+%282015%29' # noqa

RSS_YEAR_2016 = 'https://hattiesburg.legistar.com/Feed.ashx?M=Calendar&ID=2857417&GUID=0f83b996-05bd-4342-be1c-fc0e267f8dcb&Mode=2016&Title=City+of+Hattiesburg%2c+MS+-+Calendar+%282016%29' # noqa

RSS_YEAR_2017 = 'https://hattiesburg.legistar.com/Feed.ashx?M=Calendar&ID=5663030&GUID=041c7c03-3032-40d2-a64b-2ad48300330f&Mode=2017&Title=City+of+Hattiesburg%2c+MS+-+Calendar+(2017)' # noqa

RSS_YEAR_2018 = 'https://hattiesburg.legistar.com/Feed.ashx?M=Calendar&ID=5663046&GUID=81404e43-eb5a-4a99-808a-8b261de92c2b&Mode=2018&Title=City+of+Hattiesburg%2c+MS+-+Calendar+(2018)' # noqa


RSS_YEARLY_FEEDS = {
    2011: RSS_YEAR_2011,
    2012: RSS_YEAR_2012,
    2013: RSS_YEAR_2013,
    2014: RSS_YEAR_2014,
    2015: RSS_YEAR_2015,
    2016: RSS_YEAR_2016,
    2017: RSS_YEAR_2017,
    2018: RSS_YEAR_2018}
